#!/bin/bash
#
# Restores a Full Backup of Nagios XI
# Copyright (c) 2008-2020 Nagios Enterprises, LLC. All rights reserved.
#

# Change this password if your root password is different than "nagiosxi"
# MySQL root password
themysqlpass="nagiosxi"

# Tests mysql connection by opening a connection and selecting the DB we want to use
test_mysql_connection() {
    local db_host="$1"
    local db_port="$2"
    local db_username="$3"
    local db_password="$4"
    db_error=$(mysql -h "$db_host" --port="$db_port" -u $db_username --password=$db_password -e "show databases;" 2>&1)
    return $?
}

# Test connecting to mysql to make sure we can connect, before continuing
verify_mysql_password () {
    local dbserver=$1
    local dbport=$2
    local testpassword=$3

    x=1

    while [ $x -le 5 ]; do
        test_mysql_connection "${dbserver}" "${dbport}" "root" "${testpassword}"

        if [ $? == 1 ]; then
            echo "ERROR: Could not connect to ${dbserver}:${dbport} with root password supplied."
            read -s -r -p "Please enter the MySQL root password: " testpassword
            echo ""
        else
            mysqlpass="$testpassword"
            break
        fi

        if [ $x -eq 5 ]; then
            echo "ERROR: Aborting restore: Could not connect to MySQL."
            echo "${db_error}"
            exit 1
        fi

        x=$((x+1))
    done
}

echoerror() { echo "$@" 1>&2; }

get_or_set_correct_password() {
    user="$1"
    dbserver="$2"
    dbport="$3"
    backup="$4"
    current="$5"
    debug=$6

    password="$current"

    if [[ "$debug" == true ]]; then
        echoerror "Syncronizing database passwords for MySQL user $user..."
    fi

    # I'm assuming the user and port and other configs from the backup database are correct
    if [ "$backup" != "$current" ]; then
        if [[ "$debug" == true ]]; then
            echoerror "Database passwords for MySQL user $user unsyncronized..."
        fi
        if ! test_mysql_connection "$dbserver" "$dbport" "$user" "${current}"; then
            if test_mysql_connection "$dbserver" "$dbport" "$user" "${backup}"; then
                if [[ "$debug" == true ]]; then
                    echoerror "Backup password correct"
                fi
                password="$backup"
            else
                if [[ "$debug" == true ]]; then
                    echoerror "Backup and current passwords both incorrect. Resetting MySQL password to current password..."
                fi

                ip=$(mysql -h $dbserver --port="$dbport" -u root -p$mysqlpass -sse "SELECT host FROM mysql.user WHERE user='$user'")
                if [ -z "$ip" ]; then
                    ip=$(ip addr | grep global | grep -m 1 'inet' | awk '/inet[^6]/{print substr($2,0)}' | sed 's|/.*||')
                fi
                if [ -z "$ip" ]; then
                    ip=$(ip addr | grep global | grep -m 1 'inet' | awk '/inet6/{print substr($2,0)}' | sed 's|/.*||')
                fi

                config='[client]\nhost='"${dbserver}"'\nport='"${dbport}"'\nuser=root\npassword='"${mysqlpass}"

                mysqlversion=$(mysql --defaults-extra-file=<(echo -e $config) -BNe "select @@version;" | cut -d '-' -f 1)
                res=$?
                if [ $res != 0 ]; then
                    echo "Error determining MySql version.  Exiting..."
                    exit 1
                fi

                if [[ $(echo "$mysqlversion 5.7" | tr " " "\n" | sort --version-sort | head -n 1) = 5.7 ]]; then 
                    mysql --defaults-extra-file=<(echo -e $config) -e "ALTER USER '$user'@'$ip' IDENTIFIED BY '${password}';"
                else
                    mysql --defaults-extra-file=<(echo -e $config) -e "SET PASSWORD FOR '$user'@'$ip' = PASSWORD('${password}');"
                fi
            fi
        else
            if [[ "$debug" == true ]]; then
                echoerror "Current password correct"
            fi
        fi
    fi

    echo -n "$password"
}

# Make sure we have the backup file
if [ $# != 1 ]; then
    echo "Usage: $0 <backupfile>"
    echo "This script restores your XI system using a previously made Nagios XI backup file."
    exit 1
fi
backupfile=$1

BASEDIR=$(dirname $(readlink -f $0))
SPECIAL_BACKUP=0

# Import Nagios XI and xi-sys.cfg config vars
. $BASEDIR/../var/xi-sys.cfg
eval $(php $BASEDIR/import_xiconfig.php)

# Must be root
me=`whoami`
if [ $me != "root" ]; then
    echo "You must be root to run this script."
    exit 1
fi

rootdir=/store/backups/nagiosxi

##############################
# MAKE SURE BACKUP FILE EXIST
##############################
if [ ! -f $backupfile ]; then
    echo "Unable to find backup file $backupfile!"
    exit 1
fi

# Get the subdir here because we need it for the following checks
subdir=$(tar tf "$backupfile" |head -1 |cut -f 1 -d /)

# Look inside the (nested) tarball to see what architecture the nagios
# executable is
if [ $backupfile == "/store/backups/nagiosxi-demo.tar.gz" ]; then
    backuparch="x86_64"
else
    backuparch=$(eval $(echo $(tar -xzOf $backupfile $subdir/nagiosxi.tar.gz | tar -xzOf - usr/local/nagiosxi/var/xi-sys.cfg |cat|grep ^arch\=));echo $arch)
fi

arch=$(uname -m)
case $arch in
    i*86 )   arch="i686" ;;
    x86_64 ) arch="x86_64" ;;
    * )      echo "Error detecting architecture."; exit 1
esac

if [ "$arch" != "$backuparch" ]; then
    echo "WARNING: you are trying to restore a $backuparch backup on a $arch system"
    echo "         Compiled plugins and other binaries will NOT be restored."
    echo
    read -r -p "Are you sure you want to continue? [y/N] " ok

    case "$ok" in
        Y | y ) : ;;
        * )     exit 1
    esac
fi

backupdist=$(eval $(echo $(tar -xzOf $backupfile $subdir/nagiosxi.tar.gz | tar -xzOf - usr/local/nagiosxi/var/xi-sys.cfg |cat|grep ^dist\=));echo $dist)

if [ "$dist" != "$backupdist" ]; then
    SPECIAL_BACKUP=1

    echo "WARNING: you are trying to restore a $backupdist backup on a $dist system"
    echo "         Compiled plugins and other binaries as well as httpd configurations"
    echo "         will NOT be restored."
    echo ""
    echo "         You will need to re-download the Nagios XI tarball, and re-install"
    echo "         the subcomponents for this system. More info here:"
    echo "         https://assets.nagios.com/downloads/nagiosxi/docs/Backing-Up-And-Restoring-Nagios-XI.pdf"
    echo ""
    read -r -p "Are you sure you want to continue? [y/N] " ok

    case "$ok" in
        Y | y ) : ;;
        * )     exit 1
    esac
fi

# Get the backupmysqlpass here because some directory change stuff happens
backupmysqlpass="$(eval $(echo $(tar -xzOf $backupfile $subdir/nagiosxi.tar.gz | tar -xzOf - usr/local/nagiosxi/var/xi-sys.cfg |cat|grep ^mysqlpass\=));echo $mysqlpass)"

##############################
# MAKE TEMP RESTORE DIRECTORY
##############################
#ts=`echo $backupfile | cut -d . -f 1`
ts=`date +%s`
echo "TS=$ts"
mydir=${rootdir}/${ts}-restore
mkdir -p $mydir
if [ ! -d $mydir ]; then
    echo "Unable to create restore directory $mydir!"
    exit 1
fi


##############################
# UNZIP BACKUP
##############################
echo "Extracting backup to $mydir..."
tar xps -f "$backupfile" -C "$mydir"

# Change to subdirectory
cd "$mydir/$subdir"

# Make sure we have some directories here...
backupdir=`pwd`
echo "In $backupdir..."
if [ ! -f nagiosxi.tar.gz ]; then
    echo "Unable to find files to restore in $backupdir"
    exit 1
fi

echo "Backup files look okay.  Preparing to restore..."


##############################
# SHUTDOWN SERVICES
##############################
echo "Shutting down services..."
$BASEDIR/manage_services.sh stop nagios
$BASEDIR/manage_services.sh stop npcd


##############################
# RESTORE DIRS
##############################
rootdir=/
echo "Restoring directories to ${rootdir}..."

# Nagios Core
echo "Restoring Nagios Core..."
if [ "$arch" == "$backuparch" ] && [ $SPECIAL_BACKUP -eq 0 ]; then
    rm -rf /usr/local/nagios
    cd $rootdir && tar xzf $backupdir/nagios.tar.gz 
else
    rm -rf /usr/local/nagios/etc /usr/local/nagios/share /usr/local/nagios/var
    cd $rootdir && tar --exclude="usr/local/nagios/bin" --exclude="usr/local/nagios/sbin" --exclude="usr/local/nagios/libexec" -xzf $backupdir/nagios.tar.gz
    cd $rootdir && tar --wildcards 'usr/local/nagios/libexec/*' -xzf $backupdir/nagios.tar.gz
fi

# Restore ramdisk if it exists
if [ -f "$backupdir/ramdisk.nagios" ]; then
    echo "Updating ramdisk configuration..."
    cp  $backupdir/ramdisk.nagios /etc/sysconfig/nagios
fi

# Nagios XI
echo "Restoring Nagios XI..."

# I've decided I always want to keep this information
mv $BASEDIR/../var/xi-sys.cfg /tmp/xi-sys.cfg

if [ "$arch" == "$backuparch" ] && [ $SPECIAL_BACKUP -eq 0 ]; then
    rm -rf /usr/local/nagiosxi
    cd $rootdir && tar xzfps $backupdir/nagiosxi.tar.gz 
else
    mv $BASEDIR/../var/certs /tmp/certs
    mv $BASEDIR/../var/keys /tmp/keys

    rm -rf /usr/local/nagiosxi
    cd $rootdir && tar xzfps $backupdir/nagiosxi.tar.gz 

    # Check for certs
    mkdir -p $BASEDIR/../var/certs
    cp -r /tmp/certs $BASEDIR/../var/

    rm -rf /tmp/certs

    # Check for keys
    mkdir -p $BASEDIR/../var/keys
    if [ -f $BASEDIR/../var/keys/xi.key ]; then
        rm -f /tmp/keys/xi.key
    fi
    cp -r /tmp/keys $BASEDIR/../var/

    rm -rf /tmp/keys
fi

# Part of always keeping the xi-sys.cfg file
cp -r /tmp/xi-sys.cfg $BASEDIR/../var/xi-sys.cfg
cp -r /tmp/xi-sys.cfg $BASEDIR/../etc/xi-sys.cfg
chown root:$nagiosgroup $BASEDIR/../etc/xi-sys.cfg
chmod 550 $BASEDIR/../etc/xi-sys.cfg
rm -f /tmp/xi-sys.cfg

# NagiosQL
if [ -d "/var/www/html/nagiosql" ]; then

    echo "Restoring NagiosQL..."
    rm -rf /var/www/html/nagiosql
    cd $rootdir && tar xzfps $backupdir/nagiosql.tar.gz

    # NagiosQL etc
    echo "Restoring NagiosQL backups..."
    rm -rf /etc/nagiosql
    cd $rootdir && tar xzfps $backupdir/nagiosql-etc.tar.gz 

fi

# NRDP
echo "Restoring NRDP backups..."
rm -rf /usr/local/nrdp
cd $rootdir && tar xzfps $backupdir/nrdp.tar.gz

# MRTG
if [ -f $backupdir/mrtg.tar.gz ]; then
    echo "Restoring MRTG..."
    rm -rf /var/lib/mrtg
    cd $rootdir && tar xzfps $backupdir/mrtg.tar.gz 
    cp -rp $backupdir/conf.d /etc/mrtg/
    cp -p $backupdir/mrtg.cfg /etc/mrtg/
    chown $apacheuser:$nagiosgroup /etc/mrtg/conf.d /etc/mrtg/mrtg.cfg
fi
cd $backupdir

# SNMP configs and MIBS
if [ -f $backupdir/etc-snmp.tar.gz ]; then
    echo "Restoring SNMP configuration files..."
    cd $rootdir && tar xzfps $backupdir/etc-snmp.tar.gz
fi
if [ -f $backupdir/usr-share-snmp.tar.gz ]; then
    echo "Restoring SNMP MIBs..."
    cd $rootdir && tar xzfps $backupdir/usr-share-snmp.tar.gz
fi

# Nagvis 
if [ -f $backupdir/nagvis.tar.gz ]; then 
    echo "Restoring Nagvis backups..." 
    rm -rf /usr/local/nagvis 
    cd $rootdir && tar xzfps $backupdir/nagvis.tar.gz 
    chown -R $apacheuser:$nagiosgroup /usr/local/nagvis 
fi 

# nagios user home
if [ -f $backupdir/home-nagios.tar.gz ]; then
    echo "Restoring nagios home dir..."
    cd $rootdir && tar xzfps $backupdir/home-nagios.tar.gz
fi

# RE-IMPORT ALL XI CFG VARS
. $BASEDIR/../var/xi-sys.cfg
php $BASEDIR/import_xiconfig.php > $BASEDIR/config.dat
. $BASEDIR/config.dat
rm -rf $BASEDIR/config.dat

###################################################################################################################################################################################
# We need to find the true root password. If the database is offloaded, we want to one from the backup tarball xi-sys.cfg, if it isn't, we want the one in the current xi-sys.cfg #
###################################################################################################################################################################################

# Making the assumption that the ndoutils dbport and server is the same as the others
if [[ "$cfg__db_info__ndoutils__dbserver" == *":"* ]]; then
    dbport=`echo "$cfg__db_info__ndoutils__dbserver" | cut -f2 -d":"`
    dbserver=`echo "$cfg__db_info__ndoutils__dbserver" | cut -f1 -d":"`
else
    dbport='3306'
    dbserver="$cfg__db_info__ndoutils__dbserver"
fi

echo "Syncronizing MySQL root password..."
if [ "$backupmysqlpass" != "$mysqlpass" ]; then
    echo "MySQL root passwords unsyncronized..."
    if ! test_mysql_connection "$dbserver" "$dbport" "root" "${mysqlpass}"; then
        if test_mysql_connection "$dbserver" "$dbport" "root" "${backupmysqlpass}"; then
            echo "Backup MySQL root password worked"
            mysqlpass="$backupmysqlpass"
        else
            verify_mysql_password "$dbserver" "$dbport" "${backupmysqlpass}"
        fi
    else
        echo "Current MySQL root password worked"
    fi
fi

# Make sure it's correct
sed -i "s/$(grep '^mysqlpass=' "$BASEDIR/../etc/xi-sys.cfg")/mysqlpass=${mysqlpass}/" $BASEDIR/../etc/xi-sys.cfg
sed -i "s/$(grep '^mysqlpass=' "$BASEDIR/../var/xi-sys.cfg")/mysqlpass=${mysqlpass}/" $BASEDIR/../var/xi-sys.cfg
sed -i "s/$(grep '^PASSWORD=' "/root/scripts/automysqlbackup")/PASSWORD=${mysqlpass}/" /root/scripts/automysqlbackup

##############################
# RESTORE DATABASES
##############################

echo "Restoring MySQL databases..."

########################
#    NDOUTILS TABLE    #
########################
if [[ "$cfg__db_info__ndoutils__dbserver" == *":"* ]]; then
    ndoutils_dbport=`echo "$cfg__db_info__ndoutils__dbserver" | cut -f2 -d":"`
    ndoutils_dbserver=`echo "$cfg__db_info__ndoutils__dbserver" | cut -f1 -d":"`
else
    ndoutils_dbport='3306'
    ndoutils_dbserver="$cfg__db_info__ndoutils__dbserver"
fi

# Sync passwords here
backup_ndoutilspass="$cfg__db_info__ndoutils__pwd"
ndoutilspass="$(get_or_set_correct_password "$cfg__db_info__ndoutils__user" "$ndoutils_dbserver" "$ndoutils_dbport" "$backup_ndoutilspass" "$ndoutilspass" true)"

# Make sure everything is correct
sed -i "s/$(grep '^ndoutilspass=' "$BASEDIR/../etc/xi-sys.cfg")/ndoutilspass=${ndoutilspass}/" $BASEDIR/../etc/xi-sys.cfg
sed -i "s/$(grep '^ndoutilspass=' "$BASEDIR/../var/xi-sys.cfg")/ndoutilspass=${ndoutilspass}/" $BASEDIR/../var/xi-sys.cfg
sed -i "s/$backup_ndoutilspass/$ndoutilspass/" "$BASEDIR/../html/config.inc.php"
sed -i "s/$backup_ndoutilspass/$ndoutilspass/" "/usr/local/nagios/etc/ndo.cfg"
if [ -f "/usr/local/nagvis/etc/nagvis.ini.php" ]; then
    sed -i "s/$backup_ndoutilspass/$ndoutilspass/" "/usr/local/nagvis/etc/nagvis.ini.php"
fi

# Test mysql and see if we can connect before continuing
verify_mysql_password "$ndoutils_dbserver" "$ndoutils_dbport" "$mysqlpass"

# $backupdir/mysql/nagios.sql"
nagios_sql_backup="$backupdir/mysql/nagios.sql"

config='[client]\nhost='"${ndoutils_dbserver}"'\nport='"${ndoutils_dbport}"'\nuser=root\npassword='"${mysqlpass}"
mysql --defaults-extra-file=<(echo -e $config) < "$nagios_sql_backup"
res=$?
if [ $res != 0 ]; then
    echo "Error restoring MySQL database 'nagios'"
    exit 1
fi

########################
#    NAGIOSQL TABLE    #
########################
if [[ "$cfg__db_info__nagiosql__dbserver" == *":"* ]]; then
    nagiosql_dbport=`echo "$cfg__db_info__nagiosql__dbserver" | cut -f2 -d":"`
    nagiosql_dbserver=`echo "$cfg__db_info__nagiosql__dbserver" | cut -f1 -d":"`
else
    nagiosql_dbport='3306'
    nagiosql_dbserver="$cfg__db_info__nagiosql__dbserver"
fi

# Sync passwords here
backup_nagiosqlpass="$cfg__db_info__nagiosql__pwd"
nagiosqlpass="$(get_or_set_correct_password "$cfg__db_info__nagiosql__user" "$nagiosql_dbserver" "$nagiosql_dbport" "$backup_nagiosqlpass" "$nagiosqlpass" true)"

# Make sure everything is correct
sed -i "s/$(grep '^nagiosqlpass=' "$BASEDIR/../etc/xi-sys.cfg")/nagiosqlpass=${nagiosqlpass}/" $BASEDIR/../etc/xi-sys.cfg
sed -i "s/$(grep '^nagiosqlpass=' "$BASEDIR/../var/xi-sys.cfg")/nagiosqlpass=${nagiosqlpass}/" $BASEDIR/../var/xi-sys.cfg
sed -i "s/$backup_nagiosqlpass/$nagiosqlpass/" "$BASEDIR/../html/config.inc.php"

# Test mysql again and see if we can connect before continuing
verify_mysql_password "$nagiosql_dbserver" "$nagiosql_dbport" "$mysqlpass"

# $backupdir/mysql/nagiosql.sql"
nagiosql_sql_backup="$backupdir/mysql/nagiosql.sql"

config='[client]\nhost='"${nagiosql_dbserver}"'\nport='"${nagiosql_dbport}"'\nuser=root\npassword='"${mysqlpass}"
mysql --defaults-extra-file=<(echo -e $config) < "$nagiosql_sql_backup"
res=$?
if [ $res != 0 ]; then
    echo "Error restoring MySQL database 'nagiosql'"
    exit 1
fi

########################
#    NAGIOSXI TABLE    #
########################
echo "Restoring Nagios XI MySQL database..."
if [[ "$cfg__db_info__nagiosxi__dbserver" == *":"* ]]; then
    nagiosxi_dbport=`echo "$cfg__db_info__nagiosxi__dbserver" | cut -f2 -d":"`
    nagiosxi_dbserver=`echo "$cfg__db_info__nagiosxi__dbserver" | cut -f1 -d":"`
else
    nagiosxi_dbport='3306'
    if [ "x$cfg__db_info__nagiosxi__dbserver" == "x" ]; then
        nagiosxi_dbserver="localhost"
    else
        nagiosxi_dbserver="$cfg__db_info__nagiosxi__dbserver"
    fi
fi

# Nagiosxi user
backup_nagiosxipass="$cfg__db_info__nagiosxi__pwd"
nagiosxipass="$(get_or_set_correct_password "$cfg__db_info__nagiosxi__user" "$nagiosxi_dbserver" "$nagiosxi_dbport" "$backup_nagiosxipass" "$nagiosxipass" true)"

# Make sure everything is correct
sed -i "s/$(grep '^nagiosxipass=' "$BASEDIR/../etc/xi-sys.cfg")/nagiosxipass=${nagiosxipass}/" $BASEDIR/../etc/xi-sys.cfg
sed -i "s/$(grep '^nagiosxipass=' "$BASEDIR/../var/xi-sys.cfg")/nagiosxipass=${nagiosxipass}/" $BASEDIR/../var/xi-sys.cfg
sed -i "s/$backup_nagiosxipass/$nagiosxipass/" "$BASEDIR/../html/config.inc.php"

# dbmaint user
backup_dbmaintpass="$cfg__db_info__nagiosxi__dbmaint_pwd"
dbmaintpass="$(get_or_set_correct_password "$cfg__db_info__nagiosxi__dbmaint_user" "$nagiosxi_dbserver" "$nagiosxi_dbport" "$backup_dbmaintpass" "$dbmaintpass" true)"

# Make sure everything is correct
sed -i "s/$(grep '^dbmaintpass=' "$BASEDIR/../etc/xi-sys.cfg")/dbmaintpass=${dbmaintpass}/" $BASEDIR/../etc/xi-sys.cfg
sed -i "s/$(grep '^dbmaintpass=' "$BASEDIR/../var/xi-sys.cfg")/dbmaintpass=${dbmaintpass}/" $BASEDIR/../var/xi-sys.cfg
sed -i "s/$backup_dbmaintpass/$dbmaintpass/" "$BASEDIR/../html/config.inc.php"

# Test mysql again and see if we can connect before continuing
verify_mysql_password "$nagiosxi_dbserver" "$nagiosxi_dbport" "$mysqlpass"

nagiosxi_sql_backup="$backupdir/mysql/nagiosxi.sql"

config='[client]\nhost='"${nagiosxi_dbserver}"'\nport='"${nagiosxi_dbport}"'\nuser=root\npassword='"${mysqlpass}"
mysql --defaults-extra-file=<(echo -e $config) < "$nagiosxi_sql_backup"
res=$?
if [ $res != 0 ]; then
    echo "Error restoring MySQL database 'nagiosxi' !"
    exit 1
fi

echo "Restarting database servers..."
$BASEDIR/manage_services.sh restart mysqld

##############################
# RESTORE CRONJOB ENTRIES
##############################
echo "Restoring Apache cronjobs..."
if [[ "$distro" == "Ubuntu" ]] || [[ "$distro" == "Debian" ]]; then
    cp $backupdir/cron/apache /var/spool/cron/crontabs/$apacheuser
else
    cp $backupdir/cron/apache /var/spool/cron/apache
fi

##############################
# RESTORE SUDOERS
##############################
# Not necessary

##############################
# RESTORE LOGROTATE
##############################
echo "Restoring logrotate config files..."
cp -rp $backupdir/logrotate/nagiosxi /etc/logrotate.d

##############################
# RESTORE APACHE CONFIG FILES
##############################
if [ $SPECIAL_BACKUP -eq 0 ]; then
    echo "Restoring Apache config files..."
    cp -rp $backupdir/httpd/nagios.conf $httpdconfdir
    cp -rp $backupdir/httpd/nagiosxi.conf $httpdconfdir
    cp -rp $backupdir/httpd/nagvis.conf $httpdconfdir
    cp -rp $backupdir/httpd/nrdp.conf $httpdconfdir
    if [ -d "/etc/apache2/sites-available" ]; then
        cp -rp $backupdir/httpd/default-ssl.conf /etc/apache2/sites-available
    else
        cp -rp $backupdir/httpd/ssl.conf $httpdconfdir
    fi
else
    echo "Skipping Apache config files restoration"
fi

##############################
# RESTART SERVICES
##############################
$BASEDIR/manage_services.sh restart httpd
$BASEDIR/manage_services.sh start npcd
$BASEDIR/manage_services.sh start nagios

##############################
# DELETE TEMP RESTORE DIRECTORY
##############################
rm -rf $mydir

echo " "
echo "==============="
echo "RESTORE COMPLETE"
echo "==============="

exit 0
